@@ -1,12 +1,11 @@
-context("HttpClient: query")
+skip_on_cran()
+skip_if_offline(url_parse(hb())$domain)
 
 test_that("query works", {
-  skip_on_cran()
-
   cli <- HttpClient$new(url = hb())
   aa <- cli$get('get', query = list(hello = "world"))
 
-  expect_is(aa, "HttpResponse")
+  expect_s3_class(aa, "HttpResponse")
   expect_match(aa$url, "hello")
   expect_match(aa$url, "world")
   expect_match(jsonlite::fromJSON(aa$parse())$url, "hello")
@@ -14,23 +13,21 @@ test_that("query works", {
 })
 
 test_that("query - multiple params of same name work", {
-  skip_on_cran()
-
   cli <- HttpClient$new(url = hb())
   aa <- cli$get('get', query = list(hello = 5, hello = 6))
 
-  expect_is(aa, "HttpResponse")
+  expect_s3_class(aa, "HttpResponse")
   expect_equal(length(gregexpr("hello", aa$url)[[1]]), 2)
   expect_equal(
-    length(gregexpr("hello", jsonlite::fromJSON(aa$parse())$url)[[1]]), 2)
+    length(gregexpr("hello", jsonlite::fromJSON(aa$parse())$url)[[1]]),
+    2
+  )
 })
 
 test_that("query - length 0 query list works", {
-  skip_on_cran()
-
   cli <- HttpClient$new(url = hb())
   aa <- cli$get('get', query = list())
 
-  expect_is(aa, "HttpResponse")
+  expect_s3_class(aa, "HttpResponse")
   expect_false(grepl("\\?", aa$url))
 })
